clear all
close all
clc

%load data from Rainpower file and sort it
data=table2array(readtable('Hillchart_r3_medBEP.xlsx','Sheet','Hy',...
                           'Range','B2:F257'));    %[etaM Q11 N11 T11 alfa]
etaM=data(:,1); Q11=data(:,2); N11=data(:,3); T11=data(:,4); alfa=data(:,5);

%remove data for negative efficiencies (irrelevant)
positive=find(etaM>=0);
etaM=etaM(positive); Q11=Q11(positive); N11=N11(positive); T11=T11(positive); alfa=alfa(positive);

%find rated values according to best efficiency point (BEP)
etaM_R=max(etaM);
pos=find(etaM==etaM_R);
Q11_R=Q11(pos); N11_R=N11(pos); T11_R=T11(pos); alfa_R=alfa(pos);
speednumber=N11_R*(pi/30)*sqrt(Q11_R)/(2*9.82)^(3/4);

%non-dimensionalize by dividing by rated values
eta=etaM./etaM_R; q=Q11./Q11_R; n=N11./N11_R; t=T11./T11_R; y=sind(alfa)./sind(alfa_R);

%--------------------------------------------------------------------------
% common for plot 1 - 4:
%--------------------------------------------------------------------------

y_query=linspace(0.2,1.4,13);               %y=1.0 included
dn=0.01;
nhalf1=linspace(min(n),1,ceil((1-min(n))/dn));
nhalf2=linspace(1,max(n),ceil((max(n)-1)/dn));
n_query=[nhalf1 nhalf2(2:end)];             %n=1.0 included

[Y,N]=meshgrid(y_query,n_query);            %each column in Y const opening degree
                                            %each row in N const speed
                                                 
%--------------------------------------------------------------------------
% 1. Flow - speed plot at different opening degrees and const rated head
%--------------------------------------------------------------------------
F1=scatteredInterpolant(y,n,q,'natural');   %create interpolant that fits a surface on the form q=F1(y,n)
Q=F1(Y,N);                                  %evaluate F1 at query points specified by Y,N matrices

figure(1)
axis([0 2 0 2])
xlabel('$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
ylabel('q [-]','Interpreter','latex','FontSize',16)
title('Measurements: Flow - speed at h=1','Interpreter','latex','FontSize',16)
hold on

%plot one curve per opening degree
for i=1:length(y_query)
    Q(:,i)=smooth(Q(:,i),10);               %comment out for raw data
    txt=['y = ',num2str(y_query(i))];
    plot(N(2:end-1,i),Q(2:end-1,i),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show

% find dq/dn at y=n=q=h=1
pos_n=find(n_query==1);
pos_y=find(y_query==1);
q_test=Q(pos_n,pos_y)                       %should be 1

% using a central difference scheme
dqdn=(Q(pos_n+1,pos_y)-Q(pos_n-1,pos_y))/(N(pos_n+1,pos_y)-N(pos_n-1,pos_y));
dqdn_2=(Q(pos_n+2,pos_y)-Q(pos_n-2,pos_y))/(N(pos_n+2,pos_y)-N(pos_n-2,pos_y));
dqdn_3=(Q(pos_n+3,pos_y)-Q(pos_n-3,pos_y))/(N(pos_n+3,pos_y)-N(pos_n-3,pos_y));

% using inbuildt gradient function
f=gradient(Q(:,pos_y));
dqdn_check=f(pos_n)/(0.5*(n_query(pos_n+1)-n_query(pos_n-1)));

%--------------------------------------------------------------------------
% 2. Torque - speed plot at different opening degrees and const rated head
%--------------------------------------------------------------------------
F2=scatteredInterpolant(y,n,t,'natural');   %create interpolant that fits a surface on the form t=F2(y,n)
T=F2(Y,N);                                  %evaluate F2 at query points specified by Y,N matrices

figure(2)
axis([0 2 0 2])
xlabel('$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
ylabel('t [-]','Interpreter','latex','FontSize',16)
title('Measurements: Torque - speed at h=1','Interpreter','latex','FontSize',16)
hold on

%plot one curve per opening degree
for i=1:length(y_query)
    T(:,i)=smooth(T(:,i),10);               %comment out for raw data
    txt=['y = ',num2str(y_query(i))];
    plot(N(2:end-1,i),T(2:end-1,i),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show

% find dt/dn at y=n=t=h=1
pos_n=find(n_query==1);
pos_y=find(y_query==1);
t_test=T(pos_n,pos_y)                       %should be 1

% using a central difference scheme
dtdn=(T(pos_n+1,pos_y)-T(pos_n-1,pos_y))/(N(pos_n+1,pos_y)-N(pos_n-1,pos_y));
dtdn_2=(T(pos_n+2,pos_y)-T(pos_n-2,pos_y))/(N(pos_n+2,pos_y)-N(pos_n-2,pos_y));
dtdn_3=(T(pos_n+3,pos_y)-T(pos_n-3,pos_y))/(N(pos_n+3,pos_y)-N(pos_n-3,pos_y));

% using inbuildt gradient function
f=gradient(T(:,pos_y));
dtdn_check=f(pos_n)/(0.5*(n_query(pos_n+1)-n_query(pos_n-1)));

%------------------------------------------------------------------------------
% 3. Mech. power - speed plot at different opening degrees and const rated head
%------------------------------------------------------------------------------
P=T.*N;

figure(3)
axis([0 2 0 1.6])
xlabel('$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
ylabel('p = t*$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
title('Measurements: Output power - speed at h=1','Interpreter','latex','FontSize',16)
hold on

%plot one curve per opening degree
for i=1:length(y_query)
    P(:,i)=smooth(P(:,i));                  %comment out for raw data
    txt=['y = ',num2str(y_query(i))];
    plot(N(2:end-1,i),P(2:end-1,i),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show


%-----------------------------------------------------------------------------
% 4. Efficiency - speed plot at different opening degrees and const rated head
%-----------------------------------------------------------------------------
F4=scatteredInterpolant(y,n,eta,'natural'); %create interpolant that fits a surface on the form eta=F4(y,n)                                     
ETA=F4(Y,N);                                %evaluate F4 at query points specified by Y,N matrices

figure(4)
axis([0 2 0 1.1])
xlabel('$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
title('Measurements: Efficiency - speed at h=1','Interpreter','latex','FontSize',16)
hold on

%plot one curve per opening degree
for i=1:length(y_query)
    ETA(:,i)=smooth(ETA(:,i));              %comment out for raw data
    txt=['y = ',num2str(y_query(i))];
    plot(N(2:end-1,i),ETA(2:end-1,i),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show


%--------------------------------------------------------------------------
% common for plot 5 - 8:
%--------------------------------------------------------------------------

n_query=linspace(0.8,1.3,11);               %n=1.0 included
dy=0.01;
yhalf1=linspace(min(y),1,ceil((1-min(y))/dy));
yhalf2=linspace(1,max(y),ceil((max(y)-1)/dy));
y_query=[yhalf1 yhalf2(2:end)];             %y=1.0 included  

[N,Y]=meshgrid(n_query,y_query);            %each column in N const speed
                                            %each row in Y const opening degree

%--------------------------------------------------------------------------
% 5. Flow - opening degree plot at different speeds and const rated head
%--------------------------------------------------------------------------
F5=scatteredInterpolant(n,y,q,'natural');   %create interpolant that fits a surface on the form q=F5(n,y)                                        
Q=F5(N,Y);                                  %evaluate F5 at query points specified by N,Y matrices

figure(5)
axis([0 2 0 2])
xlabel('y [-]','Interpreter','latex','FontSize',16)
ylabel('q [-]','Interpreter','latex','FontSize',16)
title('Measurements: Flow - opening degree at h=1','Interpreter','latex','FontSize',16)
hold on

%plot one curve per speed
for i=2:length(n_query)
    Q(:,i)=smooth(Q(:,i),10);               %comment out for raw data
    txt=['$\tilde{\omega} = $',num2str(n_query(i))];
    plot(Y(2:end-1,i),Q(2:end-1,i),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show

% find dq/dy at n=y=q=h=1
pos_y=find(y_query==1);
pos_n=find(n_query==1);
q_test=Q(pos_y,pos_n)                       %should be 1

% using a central difference scheme
dqdy=(Q(pos_y+1,pos_n)-Q(pos_y-1,pos_n))/(Y(pos_y+1,pos_n)-Y(pos_y-1,pos_n));
dqdy_2=(Q(pos_y+2,pos_n)-Q(pos_y-2,pos_n))/(Y(pos_y+2,pos_n)-Y(pos_y-2,pos_n));
dqdy_3=(Q(pos_y+3,pos_n)-Q(pos_y-3,pos_n))/(Y(pos_y+3,pos_n)-Y(pos_y-3,pos_n));

% using inbuildt gradient function
f=gradient(Q(:,pos_n));
dqdy_check=f(pos_y)/(0.5*(y_query(pos_y+1)-y_query(pos_y-1)));

%--------------------------------------------------------------------------
% 6. Torque - opening degree plot at different speeds and const rated head
%--------------------------------------------------------------------------
F6=scatteredInterpolant(n,y,t,'natural');   %create interpolant that fits a surface on the form t=F6(n,y)                                    
T=F6(N,Y);                                  %evaluate F6 at query points specified by N,Y matrices

figure(6)
axis([0 2 0 2])
xlabel('y [-]','Interpreter','latex','FontSize',16)
ylabel('t [-]','Interpreter','latex','FontSize',16)
title('Measurements: Torque - opening degree at h=1','Interpreter','latex','FontSize',16)
hold on

%plot one curve per speed
for i=2:length(n_query)
    T(:,i)=smooth(T(:,i),8);                %comment out for raw data
    txt=['$\tilde{\omega} = $',num2str(n_query(i))];
    plot(Y(2:end-2,i),T(2:end-2,i),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10,'Location','northwest')
end

hold off
legend show

% find dt/dy at n=y=t=h=1
pos_y=find(y_query==1);
pos_n=find(n_query==1);
t_test=T(pos_y,pos_n)                       %should be 1

% using a central difference scheme
dtdy=(T(pos_y+1,pos_n)-T(pos_y-1,pos_n))/(Y(pos_y+1,pos_n)-Y(pos_y-1,pos_n));
dtdy_2=(T(pos_y+2,pos_n)-T(pos_y-2,pos_n))/(Y(pos_y+2,pos_n)-Y(pos_y-2,pos_n));
dtdy_3=(T(pos_y+3,pos_n)-T(pos_y-3,pos_n))/(Y(pos_y+3,pos_n)-Y(pos_y-3,pos_n));

% using inbuildt gradient function
f=gradient(T(:,pos_n));
dtdy_check=f(pos_y)/(0.5*(y_query(pos_y+1)-y_query(pos_y-1)));

%------------------------------------------------------------------------------
% 7. Mech. power - opening degree plot at different speeds and const rated head
%------------------------------------------------------------------------------
P=T.*N;

figure(7)
axis([0 2 0 2])
xlabel('y [-]','Interpreter','latex','FontSize',16)
ylabel('p = t*$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
title('Measurements: Output power - opening degree at h=1','Interpreter','latex','FontSize',16)
hold on

%plot one curve per speed
for i=2:length(n_query)
    P(:,i)=smooth(P(:,i));                  %comment out for raw data
    txt=['$\tilde{\omega} = $',num2str(n_query(i))];
    plot(Y(2:end-1,i),P(2:end-1,i),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show

%-----------------------------------------------------------------------------
% 8. Efficiency - opening degree plot at different speeds and const rated head
%-----------------------------------------------------------------------------
F8=scatteredInterpolant(n,y,eta,'natural'); %create interpolant that fits a surface on the form eta=F8(n,y)                                   
ETA=F8(N,Y);                                %evaluate F8 at query points specified by N,Y matrices

figure(8)
axis([0 2 0 1.1])
xlabel('y [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
title('Measurements: Efficiency - opening degree at h=1','Interpreter','latex','FontSize',16)
hold on

%plot one curve per speed
for i=2:length(n_query)
    ETA(:,i)=smooth(ETA(:,i));              %comment out for raw data
    txt=['$\tilde{\omega} = $',num2str(n_query(i))];
    plot(Y(2:end-1,i),ETA(2:end-1,i),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10,'Location','southeast')
end

hold off
legend show


%--------------------------------------------------------------------------
% common for plot 9 - 11:
%--------------------------------------------------------------------------

n_query=linspace(0.8,1.3,11);               %n=1.0 included
dq=0.005;
qhalf1=linspace(min(q),1,ceil((1-min(q))/dq));
qhalf2=linspace(1,max(q),ceil((max(q)-1)/dq));
q_query=[qhalf1 qhalf2(2:end)];             %q=1.0 included  

[N,Q]=meshgrid(n_query,q_query);            %each column in N const speed
                                            %each row in Q const flow
     
%--------------------------------------------------------------------------
% 9. Torque - flow plot at different speeds and const rated head
%--------------------------------------------------------------------------
F9=scatteredInterpolant(n,q,t,'natural');   %create interpolant that fits a surface on the form t=F9(n,q)                             
T=F9(N,Q);                                  %evaluate F9 at query points specified by N,Q matrices

figure(9)
axis([0 2 0 2])
xlabel('q [-]','Interpreter','latex','FontSize',16)
ylabel('t [-]','Interpreter','latex','FontSize',16)
title('Measurements: Torque - flow at h=1','Interpreter','latex','FontSize',16)
hold on

%plot one curve per speed
for i=2:length(n_query)
    T(:,i)=smooth(T(:,i));                  %comment out for raw data
    txt=['$\tilde{\omega} = $',num2str(n_query(i))];
    plot(Q(2:end-1,i),T(2:end-1,i),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show

%--------------------------------------------------------------------------
% 10. Mech. power - flow plot at different speeds and const rated head
%--------------------------------------------------------------------------
P=T.*N;

figure(10)
axis([0 2 0 2])
xlabel('q [-]','Interpreter','latex','FontSize',16)
ylabel('p = t*$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
title('Measurements: Output power - flow at h=1','Interpreter','latex','FontSize',16)
hold on

%plot one curve per speed
for i=2:length(n_query)
    P(:,i)=smooth(P(:,i));                  %comment out for raw data
    txt=['$\tilde{\omega} = $',num2str(n_query(i))];
    plot(Q(2:end-1,i),P(2:end-1,i),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show

%--------------------------------------------------------------------------
% 11. Efficiency - flow plot at different speeds and const rated head
%--------------------------------------------------------------------------
F11=scatteredInterpolant(n,q,eta,'natural'); %create interpolant that fits a surface on the form eta=F11(n,q)       
ETA=F11(N,Q);                                %evaluate F11 at query points specified by N,Q matrices

figure(11)
axis([0 2 0 1.1])
xlabel('q [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
title('Measurements: Efficiency - flow at h=1','Interpreter','latex','FontSize',16)
hold on

%plot one curve per speed
for i=2:length(n_query)
    ETA(:,i)=smooth(ETA(:,i));              %comment out for raw data
    txt=['$\tilde{\omega} = $',num2str(n_query(i))];
    plot(Q(2:end-1,i),ETA(2:end-1,i),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10,'Location','southeast')
end

hold off
legend show


%--------------------------------------------------------------------------
% common for plot 12 - 14:
%--------------------------------------------------------------------------

h=n.^(-2);                                  %H/H_R = (N_11R/N_11)^2 = 1/n^2, for n=n_R
q=q./n;                                     %Q/Q_R = sqrt(h)*Q_11/Q_11R = (N_11R/N_11)*(Q_11/Q_11R) = (1/n)*q, for n=n_R
t=t.*n.^(-2);                               %T/T_R = h*T_11/T_11R = (N_11R/N_11)^2*(T_11/T_11R) = (1/n^2)*t, for n=n_R
    
y_query=linspace(0.2,1.4,13);               %y=1.0 included
dh=0.01;
hhalf1=linspace(min(h),1,ceil((1-min(h))/dh));
hhalf2=linspace(1,max(h),ceil((max(h)-1)/dh));
h_query=[hhalf1 hhalf2(2:end)];             %h=1.0 included

[Y,H]=meshgrid(y_query,h_query);            %each column in Y const opening degree
                                            %each row in H const head
                                                               
%--------------------------------------------------------------------------
% 12. Flow - head plot at different opening degrees and const rated speed
%--------------------------------------------------------------------------
F12=scatteredInterpolant(y,h,q,'natural');  %create interpolant that fits a surface on the form q=F12(y,h)
Q=F12(Y,H);                                 %evaluate F12 at query points specified by Y,H matrices

figure(12)
axis([0 2 0 2])
xlabel('h [-]','Interpreter','latex','FontSize',16)
ylabel('q [-]','Interpreter','latex','FontSize',16)
title('Measurements: Flow - head at $\tilde{\omega} = 1$','Interpreter','latex','FontSize',16)
hold on

%plot one curve per opening degree
for i=1:length(y_query)
    Q(:,i)=smooth(Q(:,i),8);                %comment out for raw data
    txt=['y = ',num2str(y_query(i))];
    plot(H(2:end-2,i),Q(2:end-2,i),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show

% find dq/dh at y=h=q=n=1 
pos_h=find(h_query==1);
pos_y=find(y_query==1);
q_test=Q(pos_h,pos_y)                       %should be 1

% using a central difference scheme
dqdh=(Q(pos_h+1,pos_y)-Q(pos_h-1,pos_y))/(H(pos_h+1,pos_y)-H(pos_h-1,pos_y));
dqdh_2=(Q(pos_h+2,pos_y)-Q(pos_h-2,pos_y))/(H(pos_h+2,pos_y)-H(pos_h-2,pos_y));
dqdh_3=(Q(pos_h+3,pos_y)-Q(pos_h-3,pos_y))/(H(pos_h+3,pos_y)-H(pos_h-3,pos_y));

% using inbuildt gradient function
f=gradient(Q(:,pos_y));
dqdh_check=f(pos_h)/(0.5*(h_query(pos_h+1)-h_query(pos_h-1)));

%--------------------------------------------------------------------------
% 13. Torque - head plot at different opening degrees and const rated speed
%--------------------------------------------------------------------------
F13=scatteredInterpolant(y,h,t,'natural');  %create interpolant that fits a surface on the form t=F13(y,h)
T=F13(Y,H);                                 %evaluate F13 at query points specified by Y,H matrices

figure(13)
axis([0 2 0 2.5])
xlabel('h [-]','Interpreter','latex','FontSize',16)
ylabel('t [-]','Interpreter','latex','FontSize',16)
title('Measurements: Torque - head at $\tilde{\omega} = 1$','Interpreter','latex','FontSize',16)
hold on

%plot one curve per opening degree
for i=1:length(y_query)
    T(:,i)=smooth(T(:,i),8);                %comment out for raw data
    txt=['y = ',num2str(y_query(i))];
    plot(H(2:end-1,i),T(2:end-1,i),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show

% find dt/dh at y=h=t=n=1 
pos_h=find(h_query==1);
pos_y=find(y_query==1);
t_test=T(pos_h,pos_y)                       %should be 1

% using a central difference scheme
dtdh=(T(pos_h+1,pos_y)-T(pos_h-1,pos_y))/(H(pos_h+1,pos_y)-H(pos_h-1,pos_y));
dtdh_2=(T(pos_h+2,pos_y)-T(pos_h-2,pos_y))/(H(pos_h+2,pos_y)-H(pos_h-2,pos_y));
dtdh_3=(T(pos_h+3,pos_y)-T(pos_h-3,pos_y))/(H(pos_h+3,pos_y)-H(pos_h-3,pos_y));

% using inbuildt gradient function
f=gradient(T(:,pos_y));
dtdh_check=f(pos_h)/(0.5*(h_query(pos_h+1)-h_query(pos_h-1)));

%------------------------------------------------------------------------------
% 14. Efficiency - head plot at different opening degrees and const rated speed
%------------------------------------------------------------------------------
F14=scatteredInterpolant(y,h,eta,'natural'); %create interpolant that fits a surface on the form eta=F14(y,h)
ETA=F14(Y,H);                                %evaluate F14 at query points specified by Y,H matrices

figure(14)
axis([0 2 0 1.1])
xlabel('h [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
title('Measurements: Efficiency - head at $\tilde{\omega} = 1$','Interpreter','latex','FontSize',16)
hold on

%plot one curve per opening degree
for i=1:length(y_query)
    ETA(:,i)=smooth(ETA(:,i));              %comment out for raw data
    txt=['y = ',num2str(y_query(i))];
    plot(H(2:end-1,i),ETA(2:end-1,i),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show


%--------------------------------------------------------------------------
% Finding the a_ij characteristics at BEP and their error
%--------------------------------------------------------------------------
a_11=dqdh;
a_12=dqdy;
a_13=dqdn;
a_21=dtdh/a_11;
a_22=dtdy - a_21*a_12;
a_23=dtdn - a_21*a_13;

a_11=round(a_11,2); a_12=round(a_12,2); a_13=round(a_13,2); a_21=round(a_21,2); a_22=round(a_22,2); a_23=round(a_23,2); 

check=a_21+a_22                             %should be ~1

% error of analytical a_ij wrt measurements
a_ij=table2array(readtable('Hillchart_r3_medBEP.xlsx','Sheet','Turbin_design',...
                           'Range','U68:U74'));
err_a_11=abs(round(a_ij(1),2)-a_11);
err_a_12=abs(round(a_ij(2),2)-a_12);
err_a_13=abs(round(a_ij(3),2)-a_13);
err_a_21=abs(round(a_ij(4),2)-a_21);
err_a_22=abs(round(a_ij(5),2)-a_22);
err_a_23=abs(round(a_ij(6),2)-a_23);