% Non-linear turbine model with eta_i implementation specified in eta_i
% functions at the bottom of script - high head turbine

% clear all
close all
clc

% Design parameters from excel sheet - choose turbine by specifying column, run its 'eta_i_xxx' script before this script
 design=table2array(readtable('Hillchart_r3_medBEP.xlsx','Sheet','Turbin_design','Range','U9:U80'));

% necessary model input:
 alfa_1R=design(49,1); sigma=design(53,1); psi=design(54,1); xi=design(56,1);


%--------------------------------------------------------------------------
% common for plot 1 - 4:
%--------------------------------------------------------------------------
omega_query=linspace(0,1.54,155);                   %omega=1 included
y_query=linspace(0.2,1.4,13);                       %y=1 included
h=1;


%--------------------------------------------------------------------------
% 1. Flow - speed plot at different opening degrees and const rated head
%--------------------------------------------------------------------------
figure(1)
axis([0 2 0 2])
xlabel('$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
ylabel('q [-]','Interpreter','latex','FontSize',16)
title('Model with $\eta_i(q)$: Flow - speed at h=1','Interpreter','latex','FontSize',16)
hold on

q=zeros(length(y_query),length(omega_query));       %each row const opening, each column const speed
for i=1:length(y_query)
    q(i,:)=y_query(i)*sqrt(h-sigma*(omega_query.^2-1));
    
    %plot one curve per opening degree
    txt=['y = ',num2str(y_query(i))];
    plot(omega_query,q(i,:),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show

% find dq/domega at q=omega=y=h=1
pos_omega=find(omega_query==1);
pos_y=find(y_query==1);
q_test=q(pos_y,pos_omega)                           %should be 1

% using a central difference scheme
dqdomega=(q(pos_y,pos_omega+1)-q(pos_y,pos_omega-1))/(omega_query(pos_omega+1)-omega_query(pos_omega-1));

% using inbuildt gradient function
f=gradient(q(pos_y,:));
dqdomega_check=f(pos_omega)/(0.5*(omega_query(pos_omega+1)-omega_query(pos_omega-1)));


%--------------------------------------------------------------------------
% 2. Torque - speed plot at different opening degrees and const rated head
%--------------------------------------------------------------------------
figure(2)
axis([0 2 0 2])
xlabel('$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
ylabel('t [-]','Interpreter','latex','FontSize',16)
title('Model with $\eta_i(q)$: Torque - speed at h=1','Interpreter','latex','FontSize',16)
hold on

t=zeros(length(y_query),length(omega_query));       %each row const opening, each column const speed
for i=1:length(y_query)
    alfa_1=asin(y_query(i)*sin(alfa_1R*pi/180));    %radians
    m_S=xi*q(i,:)./y_query(i)*(cos(alfa_1)+tan(alfa_1R*pi/180)*sin(alfa_1));
    t(i,:)=eta_i(q(i,:)).*q(i,:).*(m_S - psi.*omega_query);
    
    %plot one curve per opening degree
    txt=['y = ',num2str(y_query(i))];
    plot(omega_query,t(i,:),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show

% find dt/domega at t=omega=y=h=1
pos_omega=find(omega_query==1);
pos_y=find(y_query==1);
t_test=t(pos_y,pos_omega)                           %should be 1

% using a central difference scheme
dtdomega=(t(pos_y,pos_omega+1)-t(pos_y,pos_omega-1))/(omega_query(pos_omega+1)-omega_query(pos_omega-1));

% using inbuildt gradient function
f=gradient(t(pos_y,:));
dtdomega_check=f(pos_omega)/(0.5*(omega_query(pos_omega+1)-omega_query(pos_omega-1)));


%------------------------------------------------------------------------------
% 3. Mech. power - speed plot at different opening degrees and const rated head
%------------------------------------------------------------------------------
figure(3)
axis([0 2 0 1.6])
xlabel('$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
ylabel('p = t*$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
title('Model with $\eta_i(q)$: Output power - speed at h=1','Interpreter','latex','FontSize',16)
hold on

p=zeros(length(y_query),length(omega_query));       %each row const opening, each column const speed
for i=1:length(y_query)
    p(i,:)=t(i,:).*omega_query;
    
    %plot one curve per opening degree
    txt=['y = ',num2str(y_query(i))];
    plot(omega_query,p(i,:),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show


%-----------------------------------------------------------------------------
% 4. Efficiency - speed plot at different opening degrees and const rated head
%-----------------------------------------------------------------------------
figure(4)
axis([0 2 0 1.1])
xlabel('$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
title('Model with $\eta_i(q)$: Efficiency - speed at h=1','Interpreter','latex','FontSize',16)
hold on

eta=zeros(length(y_query),length(omega_query));     %each row const opening, each column const speed
for i=1:length(y_query)
    
    eta(i,:)=(t(i,:).*omega_query)./(q(i,:).*1);    %since h=1
    
    %plot one curve per opening degree
    txt=['y = ',num2str(y_query(i))];
    plot(omega_query,eta(i,:),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show



%--------------------------------------------------------------------------
% common for plot 5 - 8:
%--------------------------------------------------------------------------

y_query=linspace(0,1.6,81);                         %y=1 included
omega_query=linspace(0.85,1.3,10);                  %omega=1 included
h=1;


%--------------------------------------------------------------------------
% 5. Flow - opening degree plot at different speeds and const rated head
%--------------------------------------------------------------------------
figure(5)
axis([0 2 0 2])
xlabel('y [-]','Interpreter','latex','FontSize',16)
ylabel('q [-]','Interpreter','latex','FontSize',16)
title('Model with $\eta_i(q)$: Flow - opening degree at h=1','Interpreter','latex','FontSize',16)
hold on

q=zeros(length(y_query),length(omega_query));       %each row const opening, each column const speed
for j=1:length(omega_query)
    q(:,j)=y_query.*sqrt(h-sigma*(omega_query(j)^2-1));
    
    %plot one curve per speed
    txt=['$\tilde{\omega} = $',num2str(omega_query(j))];
    plot(y_query,q(:,j),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show

% find dq/dy at q=y=omega=h=1
pos_y=find(y_query==1);
pos_omega=find(omega_query==1);
q_test=q(pos_y,pos_omega)                           %should be 1

% using a central difference scheme
dqdy=(q(pos_y+1,pos_omega)-q(pos_y-1,pos_omega))/(y_query(pos_y+1)-y_query(pos_y-1));

% using inbuildt gradient function
f=gradient(q(:,pos_omega));
dqdy_check=f(pos_y)/(0.5*(y_query(pos_y+1)-y_query(pos_y-1)));


%--------------------------------------------------------------------------
% 6. Torque - opening degree plot at different speeds and const rated head
%--------------------------------------------------------------------------
figure(6)
axis([0 2 0 2])
xlabel('y [-]','Interpreter','latex','FontSize',16)
ylabel('t [-]','Interpreter','latex','FontSize',16)
title('Model with $\eta_i(q)$: Torque - opening degree at h=1','Interpreter','latex','FontSize',16)
hold on

t=zeros(length(y_query),length(omega_query));       %each row const opening, each column const speed
alfa_1=asin(y_query.*sin(alfa_1R*pi/180));          %radians

for j=1:length(omega_query)
    m_S=xi*((q(:,j)')./y_query).*(cos(alfa_1)+tan(alfa_1R*pi/180)*sin(alfa_1));
    t(:,j)=eta_i((q(:,j)')).*(q(:,j)').*(m_S - psi.*omega_query(j));
    
    %plot one curve per speed
    txt=['$\tilde{\omega} = $',num2str(omega_query(j))];
    plot(y_query,t(:,j),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10,'Location','northwest')
end

hold off
legend show

% find dt/dy at t=y=omega=h=1
pos_omega=find(omega_query==1);
pos_y=find(y_query==1);
t_test=t(pos_y,pos_omega)                           %should be 1

% using a central difference scheme
dtdy=(t(pos_y+1,pos_omega)-t(pos_y-1,pos_omega))/(y_query(pos_y+1)-y_query(pos_y-1));

% using inbuildt gradient function
f=gradient(t(:,pos_omega));
dtdy_check=f(pos_y)/(0.5*(y_query(pos_y+1)-y_query(pos_y-1)));


%------------------------------------------------------------------------------
% 7. Mech. power - opening degree plot at different speeds and const rated head
%------------------------------------------------------------------------------
figure(7)
axis([0 2 0 2])
xlabel('y [-]','Interpreter','latex','FontSize',16)
ylabel('p = t*$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
title('Model with $\eta_i(q)$: Output power - opening degree at h=1','Interpreter','latex','FontSize',16)
hold on

p=zeros(length(y_query),length(omega_query));       %each row const opening, each column const speed

for j=1:length(omega_query)
    p(:,j)=t(:,j).*omega_query(j);
    
    %plot one curve per speed
    txt=['$\tilde{\omega} = $',num2str(omega_query(j))];
    plot(y_query,p(:,j),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show


%-----------------------------------------------------------------------------
% 8. Efficiency - opening degree plot at different speeds and const rated head
%-----------------------------------------------------------------------------
figure(8)
axis([0 2 0 1.1])
xlabel('y [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
title('Model with $\eta_i(q)$: Efficiency - opening degree at h=1','Interpreter','latex','FontSize',16)
hold on

eta=zeros(length(y_query),length(omega_query));     %each row const opening, each column const speed
for j=1:length(omega_query)
    
    eta(:,j)=(t(:,j).*omega_query(j))./(q(:,j).*1); %since h=1
    
    %plot one curve per speed
    txt=['$\tilde{\omega} = $',num2str(omega_query(j))];
    plot(y_query,eta(:,j),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show



%--------------------------------------------------------------------------
% common for plot 9 - 11:
%--------------------------------------------------------------------------

q_query=linspace(0.02,1.56,155);                    %q=1 included
omega_query=linspace(0.85,1.3,10);                  %omega=1 included
h=1;


%--------------------------------------------------------------------------
% 9. Torque - flow plot at different speeds and const rated head
%--------------------------------------------------------------------------
figure(9)
axis([0 2 0 2])
xlabel('q [-]','Interpreter','latex','FontSize',16)
ylabel('t [-]','Interpreter','latex','FontSize',16)
title('Model with $\eta_i(q)$: Torque - flow at h=1','Interpreter','latex','FontSize',16)
hold on

t=zeros(length(q_query),length(omega_query));       %each row const flow, each column const speed
for j=1:length(omega_query)
    
    qdivy=sqrt(h-sigma*(omega_query(j)^2-1));
    y=q_query./qdivy;
    alfa_1=asin(y.*sin(alfa_1R*pi/180));            %radians
    m_S=xi*qdivy*(cos(alfa_1)+tan(alfa_1R*pi/180)*sin(alfa_1));
    
    t(:,j)=eta_i(q_query).*q_query.*(m_S - psi.*omega_query(j));
    
    %plot one curve per speed
    txt=['$\tilde{\omega} = $',num2str(omega_query(j))];
    plot(q_query,t(:,j),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show


%--------------------------------------------------------------------------
% 10. Mech. power - flow plot at different speeds and const rated head
%--------------------------------------------------------------------------
figure(10)
axis([0 2 0 2])
xlabel('q [-]','Interpreter','latex','FontSize',16)
ylabel('p = t*$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
title('Model with $\eta_i(q)$: Output power - flow at h=1','Interpreter','latex','FontSize',16)
hold on

p=zeros(length(q_query),length(omega_query));       %each row const flow, each column const speed
for j=1:length(omega_query)
    
    p(:,j)=t(:,j).*omega_query(j);
    
    %plot one curve per speed
    txt=['$\tilde{\omega} = $',num2str(omega_query(j))];
    plot(q_query,p(:,j),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show


%--------------------------------------------------------------------------
% 11. Efficiency - flow plot at different speeds and const rated head
%--------------------------------------------------------------------------
figure(11)
axis([0 2 0 1.1])
xlabel('q [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
title('Model with $\eta_i(q)$: Efficiency - flow at h=1','Interpreter','latex','FontSize',16)
hold on

eta=zeros(length(q_query),length(omega_query));     %each row const flow, each column const speed
for j=1:length(omega_query)
    
    eta(:,j)=(t(:,j).*omega_query(j))./(q_query'.*1);%since h=1
    
    %plot one curve per speed
    txt=['$\tilde{\omega} = $',num2str(omega_query(j))];
    plot(q_query,eta(:,j),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10,'Location','southeast')
end

hold off
legend show



%--------------------------------------------------------------------------
% common for plot 12 - 14:
%--------------------------------------------------------------------------

h_query=linspace(0,1.5,76);                         %h=1 included
y_query=linspace(0.2,1.4,13);                       %y=1 included
omega=1;


%--------------------------------------------------------------------------
% 12. Flow - head plot at different opening degrees and const rated speed
%--------------------------------------------------------------------------
figure(12)
axis([0 2 0 2])
xlabel('h [-]','Interpreter','latex','FontSize',16)
ylabel('q [-]','Interpreter','latex','FontSize',16)
title('Model with $\eta_i(q)$: Flow - head at $\tilde{\omega} = 1$','Interpreter','latex','FontSize',16)
hold on

q=zeros(length(y_query),length(h_query));           %each row const opening, each column const head
for i=1:length(y_query)
    q(i,:)=y_query(i)*sqrt(h_query-sigma*(omega^2-1));
    
    %plot one curve per opening degree
    txt=['y = ',num2str(y_query(i))];
    plot(h_query,q(i,:),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show

% find dq/dh at q=h=y=omega=1 
pos_h=find(h_query==1);
pos_y=find(y_query==1);
q_test=q(pos_y,pos_h)                               %should be 1

% using a central difference scheme
dqdh=(q(pos_y,pos_h+1)-q(pos_y,pos_h-1))/(h_query(pos_h+1)-h_query(pos_h-1));

% using inbuildt gradient function
f=gradient(q(pos_y,:));
dqdh_check=f(pos_h)/(0.5*(h_query(pos_h+1)-h_query(pos_h-1)));


%--------------------------------------------------------------------------
% 13. Torque - head plot at different opening degrees and const rated speed
%--------------------------------------------------------------------------
figure(13)
axis([0 2 0 2.5])
xlabel('h [-]','Interpreter','latex','FontSize',16)
ylabel('t [-]','Interpreter','latex','FontSize',16)
title('Model with $\eta_i(q)$: Torque - head at $\tilde{\omega} = 1$','Interpreter','latex','FontSize',16)
hold on

t=zeros(length(y_query),length(h_query));           %each row const opening, each column const head
for i=1:length(y_query)
    
    qdivy=sqrt(h_query-sigma*(omega^2-1));
    alfa_1=asin(y_query(i)*sin(alfa_1R*pi/180));    %radians
    m_S=xi*qdivy.*(cos(alfa_1)+tan(alfa_1R*pi/180)*sin(alfa_1));
    
    t(i,:)=eta_i(q(i,:)).*q(i,:).*(m_S - psi*omega);
    
    %plot one curve per opening degree
    txt=['y = ',num2str(y_query(i))];
    plot(h_query,t(i,:),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show

% find dt/dh at q=h=y=omega=1 
pos_h=find(h_query==1);
pos_y=find(y_query==1);
t_test=t(pos_y,pos_h)                               %should be 1 or something is wrong

% using a central difference scheme
dtdh=(t(pos_y,pos_h+1)-t(pos_y,pos_h-1))/(h_query(pos_h+1)-h_query(pos_h-1));

% using inbuildt gradient function
f=gradient(t(pos_y,:));
dtdh_check=f(pos_h)/(0.5*(h_query(pos_h+1)-h_query(pos_h-1)));


%------------------------------------------------------------------------------
% 14. Efficiency - head plot at different opening degrees and const rated speed
%------------------------------------------------------------------------------
figure(14)
axis([0 2 0 1.1])
xlabel('h [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
title('Model with $\eta_i(q)$: Efficiency - head at $\tilde{\omega} = 1$','Interpreter','latex','FontSize',16)
hold on

eta=zeros(length(y_query),length(h_query));         %each row const opening, each column const head
for i=1:length(y_query)
    
    eta(i,:)=(t(i,:).*1)./(q(i,:).*h_query);        %since omega=1
    
    %plot one curve per opening degree
    txt=['y = ',num2str(y_query(i))];
    plot(h_query,eta(i,:),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show



%--------------------------------------------------------------------------
% Finding the characteristics at BEP
%--------------------------------------------------------------------------

a_11=dqdh;
a_12=dqdy;
a_13=dqdomega;
a_21=dtdh/a_11;
a_22=dtdy-a_21*a_12;
a_23=dtdomega-a_21*a_13;

check=a_21+a_22                             %should be =1


%--------------------------------------------------------------------------
% uncomment the relevant eta_i(q) function
%eta_i multiplied into torque equation, should affect t-, p- and eta-plots
%q = single flow value/ vector at constant speed/ opening degree
%--------------------------------------------------------------------------

% 1. POLYNOMIAL FUNCTION
function y=eta_i(q)
    global poly_p1 poly_p2 poly_p3 poly_p4 poly_p5 poly_p6 poly_p7 poly_p8 poly_p9 poly_p10
    y = poly_p1*q.^9 + poly_p2*q.^8 + poly_p3*q.^7 + poly_p4*q.^6 + poly_p5*q.^5 + poly_p6*q.^4 + ...
        poly_p7*q.^3 + poly_p8*q.^2 + poly_p9*q + poly_p10;
end

% % 2. FOURIER SERIES FUNCTION
% function y=eta_i(q)
%     global fou_a0 fou_a1 fou_b1 fou_a2 fou_b2 fou_a3 fou_b3 fou_a4 fou_b4 fou_a5 fou_b5 fou_a6 fou_b6 fou_w
%     y = fou_a0 + fou_a1*cos(1*fou_w.*q) + fou_b1*sin(1*fou_w.*q) + fou_a2*cos(2*fou_w.*q) + fou_b2*sin(2*fou_w.*q)...
%     + fou_a3*cos(3*fou_w.*q) + fou_b3*sin(3*fou_w.*q) + fou_a4*cos(4*fou_w.*q) + fou_b4*sin(4*fou_w.*q)...
%     + fou_a5*cos(5*fou_w.*q) + fou_b5*sin(5*fou_w.*q) + fou_a6*cos(6*fou_w.*q) + fou_b6*sin(6*fou_w.*q);
% end

% % 3. TWO-TERM EXPONENTIAL FUNCTIONS
% function y=eta_i(q)
%     global expo1_a expo1_b expo1_c expo1_d expo2_a expo2_b expo2_c expo2_d
%     y=zeros(size(q));  
%     
%     for k=1:length(q)
%         if q(k)<=1
%             y(k) = expo1_a*exp(expo1_b*q(k)) + expo1_c*exp(expo1_d*q(k));
%         elseif q(k)>1
%             y(k) = expo2_a*exp(expo2_b*q(k)) + expo2_c*exp(expo2_d*q(k));
%         end
%     end
% end

% % 4. TWO-TERM POWER FUNCTIONS
% function y=eta_i(q)
%     global pow1_a pow1_b pow1_c pow2_a pow2_b pow2_c
%     y=zeros(size(q));
%     
%     for k=1:length(q)
%         if q(k)<1
%             y(k) = pow1_a*q(k)^pow1_b + pow1_c;
%         elseif q(k)>=1
%             y(k) = pow2_a*q(k)^pow2_b + pow2_c;
%         end
%     end
% end

% % 5. PARABOLA PROPOSED BY NIELSEN
% function y=eta_i(q)
%     y=q.*(2-q);
% end