clear all
close all
clc

%load data from Rainpower file and sort it
data=table2array(readtable('Hillchart_r3_medBEP.xlsx','Sheet','Mellom2','Range','L2:P305'));    %[etaM Q11 N11 T11 alfa]
etaM=data(:,1); Q11=data(:,2); N11=data(:,3); T11=data(:,4); alfa=data(:,5);

%remove data for negative efficiencies (irrelevant)
positive=find(etaM>=0);
etaM=etaM(positive); Q11=Q11(positive); N11=N11(positive); T11=T11(positive); alfa=alfa(positive);

%find rated values according to best efficiency point (BEP)
etaM_R=max(etaM);
pos1=find(etaM==etaM_R);
Q11_R=Q11(pos1); N11_R=N11(pos1); T11_R=T11(pos1); alfa_R=alfa(pos1);

%non-dimensionalize by dividing by rated values
eta=etaM./etaM_R; q=Q11./Q11_R; n=N11./N11_R; t=T11./T11_R; y=sind(alfa)./sind(alfa_R);

%model inputs for the geometrical similar turbine
model_inputs=table2array(readtable('Hillchart_r3_medBEP.xlsx','Sheet','Turbin_design','Range','T57:T65'));
alfa_1R=model_inputs(1,1); sigma=model_inputs(5,1); psi=model_inputs(6,1); xi=model_inputs(8,1);


%--------------------------------------------------------------------------
% Efficiency - flow plot at different speeds and const rated head
%--------------------------------------------------------------------------

n_query=linspace(0.9,1.7,17);               %n=1.0 included
dq=0.01;
qhalf1=linspace(min(q),1,ceil((1-min(q))/dq));
qhalf2=linspace(1,max(q),ceil((max(q)-1)/dq));
q_query=[qhalf1 qhalf2(2:end)];             %q=1.0 included 

[N,Q]=meshgrid(n_query,q_query);            %each column in N has const speed
                                            %each row in Q has const flow
                                        
F = scatteredInterpolant(n,q,eta,'natural');%create interpolant that fits a surface on the form eta=F(n,q)
ETA=F(N,Q);                                 %evaluate F at query points specified by N,Q matrices

figure(1)
xlabel('q [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
title('Medium head turbine','Interpreter','latex','FontSize',16)
hold on

%plot one curve per speed
for j=1:length(n_query)
    txt=['$\tilde{\omega} = $',num2str(n_query(j))];
    plot(Q(:,j),ETA(:,j),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10,'Location','southeast')
end

axis([0 2 0 1.05])
hold off
legend show


%--------------------------------------------------------------------------
% TESTING CURVE FITTINGS
%--------------------------------------------------------------------------

n1=1.0;                                          %change to investigate other speeds
pos2=find(n_query==n1);
q1=Q(:,pos2); eta_meas=ETA(:,pos2);

%calc model efficiency without eta_i(q) at h=1, omega=1, varying q
eta_model=xi*(cos(asin(q1.*sind(alfa_1R))) + q1.*tand(alfa_1R)*sind(alfa_1R)) - psi;

eta_i=eta_meas./eta_model;                       %bc eta_model*eta_i(q)~=eta_meas

%klipper vekk for q>1.48 fordi der begynner eta g oppover igjen
for j=1:length(q1)
   if q1(j)>1.48
      break
   end
end
q1=q1(1:j-1); eta_meas=eta_meas(1:j-1); eta_model=eta_model(1:j-1); eta_i=eta_i(1:j-1); 

eta_i_before=eta_i;                             %saving original data
eta_i=smooth(eta_i,10);                         %smooth data before curve fitting

q2=linspace(0,2,201);                           %for plotting

% q1=[q1;2]; eta_i=[eta_i;0];                     %adding (2,0) for better curve shape in q > q_limit


%--------------------------------------------------------------------------
% 1. POLYNOMIAL FUNCTION
%--------------------------------------------------------------------------

%extra weight on point (q,eta)=(1,1)
w=ones(size(eta_i));
pos3=find(q_query==1);
w(pos3)=10;                                       %specify >1

poly=fit([q1;2],[eta_i;0],'poly9','Weight',[w;1]);

% pp = (poly.p1)*q1.^8 + (poly.p2)*q1.^7 + (poly.p3)*q1.^6 + (poly.p4)*q1.^5 + (poly.p5)*q1.^4 + (poly.p6)*q1.^3 + ...
%     (poly.p7)*q1.^2 + (poly.p8)*q1 + (poly.p9);
% pp = (poly.p1)*q2.^8 + (poly.p2)*q2.^7 + (poly.p3)*q2.^6 + (poly.p4)*q2.^5 + (poly.p5)*q2.^4 + (poly.p6)*q2.^3 + ...
%     (poly.p7)*q2.^2 + (poly.p8)*q2 + (poly.p9);
pp = (poly.p1)*q2.^9 + (poly.p2)*q2.^8 + (poly.p3)*q2.^7 + (poly.p4)*q2.^6 + (poly.p5)*q2.^5 + (poly.p6)*q2.^4 + ...
    (poly.p7)*q2.^3 + (poly.p8)*q2.^2 + (poly.p9)*q2 + (poly.p10);


figure(2)
plot([q1;2],[eta_i;0],'r*',q2,eta_i_parabola(q2),'k-.',q2,pp,'k-','LineWidth',1)
legend('Data for curve fitting','$q (2-q)$','Polynomial of degree 9')
set(legend,'Interpreter','latex','FontSize',10,'Location','south')
xlabel('q [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
axis([0 2 0 1.05])

%test resulting function
syms x
p = poly.p1*x^9 + poly.p2*x^8 + poly.p3*x^7 + poly.p4*x^6 + poly.p5*x^5 + poly.p6*x^4 + poly.p7*x^3 + poly.p8*x^2 + poly.p9*x + poly.p10;
dpdx = diff(p);
poly_derivative=vpa(subs(dpdx,x,1))         %should be 0 or close to 0


%--------------------------------------------------------------------------
% 2. FOURIER SERIES FUNCTION
%--------------------------------------------------------------------------

fou=fit(q1,eta_i,'fourier5','Weight',w);

% ff = fou.a0 + fou.a1*cos(1*fou.w*q1) + fou.b1*sin(1*fou.w*q1) + fou.a2*cos(2*fou.w*q1) + fou.b2*sin(2*fou.w*q1)...
%     + fou.a3*cos(3*fou.w*q1) + fou.b3*sin(3*fou.w*q1) + fou.a4*cos(4*fou.w*q1) + fou.b4*sin(4*fou.w*q1)...
%     + fou.a5*cos(5*fou.w*q1) + fou.b5*sin(5*fou.w*q1); %+ fou.a6*cos(6*fou.w*q1) + fou.b6*sin(6*fou.w*q1);
ff = fou.a0 + fou.a1*cos(1*fou.w*q2) + fou.b1*sin(1*fou.w*q2) + fou.a2*cos(2*fou.w*q2) + fou.b2*sin(2*fou.w*q2)...
    + fou.a3*cos(3*fou.w*q2) + fou.b3*sin(3*fou.w*q2) + fou.a4*cos(4*fou.w*q2) + fou.b4*sin(4*fou.w*q2)...
    + fou.a5*cos(5*fou.w*q2) + fou.b5*sin(5*fou.w*q2); %+ fou.a6*cos(6*fou.w*q2) + fou.b6*sin(6*fou.w*q2);

figure(3)
plot(q1,eta_i,'r*',q2,eta_i_parabola(q2),'k-.',q2,ff,'k-','LineWidth',1)
legend('Data for curve fitting','$q (2-q)$','Fourier series of 5 harmonics')
set(legend,'Interpreter','latex','FontSize',10,'Location','south')
xlabel('q [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
axis([0 2 0 1.05])

%test resulting function
syms x
f = fou.a0 + fou.a1*cos(1*fou.w*x) + fou.b1*sin(1*fou.w*x) + fou.a2*cos(2*fou.w*x) + fou.b2*sin(2*fou.w*x)...
    + fou.a3*cos(3*fou.w*x) + fou.b3*sin(3*fou.w*x) + fou.a4*cos(4*fou.w*x) + fou.b4*sin(4*fou.w*x)...
    + fou.a5*cos(5*fou.w*x) + fou.b5*sin(5*fou.w*x); %+ fou.a6*cos(6*fou.w*x) + fou.b6*sin(6*fou.w*x);
dfdx = diff(f);
fou_derivative=vpa(subs(dfdx,x,1))          %should be 0 or close to 0


%--------------------------------------------------------------------------
% 3. TWO-TERM EXPONENTIAL FUNCTIONS
%    e1(q) for q<=1, e2(q) for q>=1
%--------------------------------------------------------------------------

q1_1=q1(find(q1<=1));                              %q1<=1
q1_2=q1(find(q1>=1));                              %q1>=1
eta_i_1=eta_i(find(q1<=1));
eta_i_2=eta_i(find(q1>=1));

%extra weight on point (q,eta)=(1,1)
w1=ones(size(eta_i_1)); w1(end)=1000;
w2=ones(size(eta_i_2)); w2(1)=1000;

%two exponential functions on eta-increasing and eta-decreasing intervals
expo1=fit(q1_1,eta_i_1,'exp2','Weight', w1);
expo2=fit(q1_2,eta_i_2,'exp2','Weight', w2);

% e1 = (expo1.a)*exp((expo1.b)*q1_1) + (expo1.c)*exp((expo1.d)*q1_1);
% e2 = (expo2.a)*exp((expo2.b)*q1_2) + (expo2.c)*exp((expo2.d)*q1_2);
q2_1=q2(find(q2<=1));
q2_2=q2(find(q2>=1));
e1 = (expo1.a)*exp((expo1.b)*q2_1) + (expo1.c)*exp((expo1.d)*q2_1);
e2 = (expo2.a)*exp((expo2.b)*q2_2) + (expo2.c)*exp((expo2.d)*q2_2);


figure(4)
plot(q1,eta_i,'r*',q2,eta_i_parabola(q2),'k-.',q2_1,e1,'k-',q2_2,e2,'k-','LineWidth',1)
legend('Data for curve fitting','$q (2-q)$','Exponential for $q<=1$','Exponential for $q>=1$')
set(legend,'Interpreter','latex','FontSize',10,'Location','south')
xlabel('q [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
axis([0 2 0 1.05])

%test resulting function
syms x
ee1 = expo1.a*exp(expo1.b*x) + expo1.c*exp(expo1.d*x);
dee1dx = diff(ee1);
e1_derivative=vpa(subs(dee1dx,x,1))          %should be 0 or close to 0

%test resulting function
syms x
ee2 = expo2.a*exp(expo2.b*x) + expo2.c*exp(expo2.d*x);
dee2dx = diff(ee2);
e2_derivative=vpa(subs(dee2dx,x,1))          %should be 0 or close to 0


%--------------------------------------------------------------------------
% 4. TWO-TERM POWER FUNCTIONS
%    pow1(q) for q<=1, pow2(q) for q>=1
%--------------------------------------------------------------------------

%two power functions on eta-increasing and eta-decreasing intervals
pow1=fit(q1_1,eta_i_1,'power2','Weight', w1);
pow2=fit(q1_2,eta_i_2,'power2','Weight', w2);

% p1 = (pow1.a)*q1_1.^(pow1.b) + (pow1.c);
% p2 = (pow2.a)*q1_2.^(pow2.b) + (pow2.c);
p1 = (pow1.a)*q2_1.^(pow1.b) + (pow1.c);
p2 = (pow2.a)*q2_2.^(pow2.b) + (pow2.c);

figure(5)
plot(q1,eta_i,'r*',q2,eta_i_parabola(q2),'k-.',q2_1,p1,'k-',q2_2,p2,'k-','LineWidth',1)
legend('Data for curve fitting','$q (2-q)$','Power for $q<=1$','Power for $q>=1$')
set(legend,'Interpreter','latex','FontSize',10,'Location','south')
xlabel('q [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
axis([0 2 0 1.05])

%test resulting function
syms x
pp1 = pow1.a*x^pow1.b + pow1.c;
dpp1dx = diff(pp1);
pow1_derivative=vpa(subs(dpp1dx,x,1))          %should be 0 or close to 0

%test resulting function
syms x
pp2 = pow2.a*x.^pow2.b + pow2.c;
dpp2dx = diff(pp2);
pow2_derivative=vpa(subs(dpp2dx,x,1))          %should be 0 or close to 0


%--------------------------------------------------------------------------
% Comparing eta_i(q) kurves
%--------------------------------------------------------------------------

figure(6)
plot(q1,eta_i_before,q1,eta_i,q1,eta_meas,q1,eta_model)
legend('eta meas/ eta model (not smooth)','eta meas/ eta model (smooth)','eta meas','eta model')
axis([0 2 0 1.05])

figure(7)
plot(q2,pp,q2,ff,q2_1,e1,q2_2,e2,q2_1,p1,q2_2,p2,'LineWidth',1)
legend('Polynomial of degree 9','Fourier series of 5 harmonics','Exponential for $q<=1$','Exponential for $q>=1$','Power for $q<=1$','Power for $q>=1$')
set(legend,'Interpreter','latex','FontSize',10,'Location','south')
title(['Medium head turbine, $\tilde{\omega}$ = ',num2str(n1)],'Interpreter','latex','FontSize',16)
xlabel('q [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
axis([0 2 0 1.05])

figure(8)
plot(q2,pp,q2,ff,'LineWidth',1)
legend('Polynomial of degree 9','Fourier series of 5 harmonics')
set(legend,'Interpreter','latex','FontSize',10,'Location','south')
title(['Medium head turbine, $\tilde{\omega}$ = ',num2str(n1)],'Interpreter','latex','FontSize',16)
xlabel('q [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
axis([0 2 0 1.05])

figure(9)
plot(q2_1,e1,q2_2,e2,q2_1,p1,q2_2,p2,'LineWidth',1)
legend('Exponential for $q<=1$','Exponential for $q>=1$','Power for $q<=1$','Power for $q>=1$')
set(legend,'Interpreter','latex','FontSize',10,'Location','south')
title(['Medium head turbine, $\tilde{\omega}$ = ',num2str(n1)],'Interpreter','latex','FontSize',16)
xlabel('q [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
axis([0 2 0 1.05])


%--------------------------------------------------------------------------
% Set curve coefficients global for use in eta_i(q) functions
%--------------------------------------------------------------------------
setGlobals_polynomial(poly);
setGlobals_fourier(fou);
setGlobals_exponential(expo1,expo2);
setGlobals_power(pow1,pow2);


% 1. POLYNOMIAL FUNCTION - set global variables
function setGlobals_polynomial(poly)
    global poly_p1 poly_p2 poly_p3 poly_p4 poly_p5 poly_p6 poly_p7 poly_p8 poly_p9 poly_p10
    poly_p1 = poly.p1;
    poly_p2 = poly.p2;
    poly_p3 = poly.p3;
    poly_p4 = poly.p4;
    poly_p5 = poly.p5;
    poly_p6 = poly.p6;
    poly_p7 = poly.p7;
    poly_p8 = poly.p8;
    poly_p9 = poly.p9;
    poly_p10 = poly.p10;
end

% 2. FOURIER SERIES FUNCTION - set global variables
function setGlobals_fourier(fou)
    global fou_a0 fou_a1 fou_b1 fou_a2 fou_b2 fou_a3 fou_b3 fou_a4 fou_b4 fou_a5 fou_b5 fou_w
    fou_a0 = fou.a0;
    fou_a1 = fou.a1;
    fou_b1 = fou.b1;
    fou_a2 = fou.a2;
    fou_b2 = fou.b2;
    fou_a3 = fou.a3;
    fou_b3 = fou.b3;
    fou_a4 = fou.a4;
    fou_b4 = fou.b4;
    fou_a5 = fou.a5;
    fou_b5 = fou.b5;
    fou_w = fou.w;
end

% 3. TWO-TERM EXPONENTIAL FUNCTIONS - set global variables
function setGlobals_exponential(expo1,expo2)
    global expo1_a expo1_b expo1_c expo1_d expo2_a expo2_b expo2_c expo2_d
    expo1_a = expo1.a;
    expo1_b = expo1.b;
    expo1_c = expo1.c;
    expo1_d = expo1.d;
    expo2_a = expo2.a;
    expo2_b = expo2.b;
    expo2_c = expo2.c;
    expo2_d = expo2.d;
end

% 4. TWO-TERM POWER FUNCTIONS - set global variables
function setGlobals_power(pow1,pow2)
    global pow1_a pow1_b pow1_c pow2_a pow2_b pow2_c
    pow1_a = pow1.a;
    pow1_b = pow1.b;
    pow1_c = pow1.c;
    pow2_a = pow2.a;
    pow2_b = pow2.b;
    pow2_c = pow2.c;
end


%the parabola proposed by Nielsen
function y=eta_i_parabola(x)
    y=x.*(2-x);
end