% Hill charts from Rainpower measurements - all 3 turbine types

clear all
close all
clc

de=0.025;  %for equdistant elevation curves


%--------------------------------------------------------------------------
% High head turbine
%--------------------------------------------------------------------------

%load data from Rainpower file and sort it
data=table2array(readtable('Hillchart_r3_medBEP.xlsx','Sheet','Hy',...
                 'Range','B2:F257'));    %[etaM Q11 N11 T11 alfa]
etaM=data(:,1); Q11=data(:,2); N11=data(:,3); T11=data(:,4); alfa=data(:,5);

%remove data for negative efficiencies (irrelevant)
positive=find(etaM>=0);
etaM=etaM(positive); Q11=Q11(positive); N11=N11(positive); T11=T11(positive); alfa=alfa(positive);

%find rated values according to best efficiency point (BEP)
etaM_R=max(etaM);
pos=find(etaM==etaM_R);
Q11_R=Q11(pos); N11_R=N11(pos); T11_R=T11(pos); alfa_R=alfa(pos);

%non-dimensionalize by dividing by rated values
eta=etaM./etaM_R; q=Q11./Q11_R; n=N11./N11_R; t=T11./T11_R; y=sind(alfa)./sind(alfa_R);

n_min=min(n)        % speed range lower limit
n_max=max(n)        % speed range upper limit
q_min=min(q)        % flow range lower limit
q_max=max(q)        % flow range upper limit

%--------------------------------------------------------------------------
y_query=linspace(0.2,1.4,13);                   %y=1.0 included

dn=0.01;
nhalf1=linspace(min(n),1,ceil((1-min(n))/dn));
nhalf2=linspace(1,max(n),ceil((max(n)-1)/dn));
n_query=[nhalf1 nhalf2(2:end)];                 %n=1.0 included

dq=0.01;
qhalf1=linspace(min(q),1,ceil((1-min(q))/dq));
qhalf2=linspace(1,max(q),ceil((max(q)-1)/dq));
q_query=[qhalf1 qhalf2(2:end)];                 %q=1.0 included
%--------------------------------------------------------------------------

[N,Q]=meshgrid(n_query,q_query);                %each column in N has const speed
                                                %each row in Q has const flow
F_ETA=scatteredInterpolant(n,q,eta,'natural');  %create interpolant that fits a surface on the form eta=F_ETA(n,q)       
ETA=F_ETA(N,Q);                                 %evaluate F_ETA at query points specified by N,Q matrices

for i=3:length(n_query-2)
    ETA(:,i)=smooth(ETA(:,i));                  %comment out for raw data
end

elev=[linspace(0,0.8,17) linspace(0.825,1.00,8)];
%elev=0:de:1;

figure(1)
contour(N,Q,ETA,elev,'-','LineWidth',1)
legend('$\tilde{\eta}$')
colorbar
caxis([0 1])
xlabel('$\tilde{\omega} / \sqrt{h}$ [-]','Interpreter','latex','FontSize',16)
ylabel('$q / \sqrt{h}$ [-]','Interpreter','latex','FontSize',16)
title('Measurements: Hill diagram','Interpreter','latex','FontSize',16)
hold on

[Y,N]=meshgrid(y_query,n_query);                %each column in Y has const opening degree
                                                %each row in N has const speed
F_Q=scatteredInterpolant(y,n,q,'natural');      %create interpolant that fits a surface on the form q=F_Q(y,n)
Q=F_Q(Y,N);                                     %evaluate F_Q at query points specified by Y,N matrices

for i=1:length(y_query)                         %plot one curve per opening degree
    Q(3:end-2,i)=smooth(Q(3:end-2,i));          %comment out for raw data
    txt=['y = ',num2str(y_query(i))];
    plot(N(2:end,i),Q(2:end,i),'-','LineWidth',1,'DisplayName',txt);  %exclude lowest speed and flow from plot
end

set(legend,'Interpreter','latex','FontSize',10,'Location','west')
axis([0 1.6 0 2])
hold off
legend show


%--------------------------------------------------------------------------
% Medium head turbine
%--------------------------------------------------------------------------

%load data from Rainpower file and sort it
data=table2array(readtable('Hillchart_r3_medBEP.xlsx','Sheet','Mellom2',...
                 'Range','L2:P305'));    %[etaM Q11 N11 T11 alfa]
etaM=data(:,1); Q11=data(:,2); N11=data(:,3); T11=data(:,4); alfa=data(:,5);

%remove data for N11=0 bc it gives h,q,t->Inf later
nonzero=find(N11~=0);
etaM=etaM(nonzero); Q11=Q11(nonzero); N11=N11(nonzero); T11=T11(nonzero); alfa=alfa(nonzero);

%remove data for negative efficiencies (irrelevant)
positive=find(etaM>=0);
etaM=etaM(positive); Q11=Q11(positive); N11=N11(positive); T11=T11(positive); alfa=alfa(positive);

%find rated values according to best efficiency point (BEP)
etaM_R=max(etaM);
pos=find(etaM==etaM_R);
Q11_R=Q11(pos); N11_R=N11(pos); T11_R=T11(pos); alfa_R=alfa(pos);

%non-dimensionalize by dividing by rated values
eta=etaM./etaM_R; q=Q11./Q11_R; n=N11./N11_R; t=T11./T11_R; y=sind(alfa)./sind(alfa_R);

n_min=min(n)        % speed range lower limit
n_max=max(n)        % speed range upper limit
q_min=min(q)        % flow range lower limit
q_max=max(q)        % flow range upper limit

%--------------------------------------------------------------------------
y_query=linspace(0.3,1.4,12);                   %y=1.0 included

dn=0.01;
nhalf1=linspace(min(n),1,ceil((1-min(n))/dn));
nhalf2=linspace(1,max(n),ceil((max(n)-1)/dn));
n_query=[nhalf1 nhalf2(2:end)];                 %n=1.0 included

dq=0.01;
qhalf1=linspace(min(q),1,ceil((1-min(q))/dq));
qhalf2=linspace(1,max(q),ceil((max(q)-1)/dq));
q_query=[qhalf1 qhalf2(2:end)];                 %q=1.0 included  
%--------------------------------------------------------------------------

[N,Q]=meshgrid(n_query,q_query);                %each column in N has const speed
                                                %each row in Q has const flow
F_ETA=scatteredInterpolant(n,q,eta,'natural');  %create interpolant that fits a surface on the form eta=F_ETA(n,q)       
ETA=F_ETA(N,Q);                                 %evaluate F_ETA at query points specified by N,Q matrices

for i=3:length(n_query-2)
    ETA(:,i)=smooth(ETA(:,i));                  %comment out for raw data
end

elev=[linspace(0,0.8,17) linspace(0.825,1.00,8)];
%elev=0:de:1;

figure(2)
contour(N,Q,ETA,elev,'-','LineWidth',1)
legend('$\tilde{\eta}$')
colorbar
caxis([0 1])
xlabel('$\tilde{\omega} / \sqrt{h}$ [-]','Interpreter','latex','FontSize',16)
ylabel('$q / \sqrt{h}$ [-]','Interpreter','latex','FontSize',16)
title('Measurements: Hill diagram','Interpreter','latex','FontSize',16)
hold on

[Y,N]=meshgrid(y_query,n_query);                %each column in Y has const opening degree
                                                %each row in N has const speed
F_Q=scatteredInterpolant(y,n,q,'natural');      %create interpolant that fits a surface on the form q=F_Q(y,n)
Q=F_Q(Y,N);                                     %evaluate F_Q at query points specified by Y,N matrices

for i=1:length(y_query)                         %plot one curve per opening degree
    Q(3:end-2,i)=smooth(Q(3:end-2,i));          %comment out for raw data
    txt=['y = ',num2str(y_query(i))];
    plot(N(3:end,i),Q(3:end,i),'-','LineWidth',1,'DisplayName',txt);  %exclude lowest speed and flow from plot
end

set(legend,'Interpreter','latex','FontSize',10,'Location','west')
axis([0 1.8 0 1.8])
hold off
legend show


%--------------------------------------------------------------------------
% Low head turbine
%--------------------------------------------------------------------------

%load data from Rainpower file and sort it
data=table2array(readtable('Hillchart_r3_medBEP.xlsx','Sheet','Lavtrykk',...
                           'Range','B2:F119'));    %[etaM Q11 N11 T11 alfa]
etaM=data(:,1); Q11=data(:,2); N11=data(:,3); T11=data(:,4); alfa=data(:,5);

%remove data for negative efficiencies (irrelevant)
positive=find(etaM>=0);
etaM=etaM(positive); Q11=Q11(positive); N11=N11(positive); T11=T11(positive); alfa=alfa(positive);

%find rated values according to best efficiency point (BEP)
etaM_R=max(etaM);
pos=find(etaM==etaM_R);
Q11_R=Q11(pos); N11_R=N11(pos); T11_R=T11(pos); alfa_R=alfa(pos);

%non-dimensionalize by dividing by rated values
eta=etaM./etaM_R; q=Q11./Q11_R; n=N11./N11_R; t=T11./T11_R; y=sind(alfa)./sind(alfa_R);

n_min=min(n)        % speed range lower limit
n_max=max(n)        % speed range upper limit
q_min=min(q)        % flow range lower limit
q_max=max(q)        % flow range upper limit

%--------------------------------------------------------------------------
y_query=linspace(0.3,1.3,11);                   %y=1.0 included

dn=0.01;
nhalf1=linspace(min(n),1,ceil((1-min(n))/dn));
nhalf2=linspace(1,max(n),ceil((max(n)-1)/dn));
n_query=[nhalf1 nhalf2(2:end)];                 %n=1.0 included

dq=0.01;
qhalf1=linspace(min(q),1,ceil((1-min(q))/dq));
qhalf2=linspace(1,max(q),ceil((max(q)-1)/dq));
q_query=[qhalf1 qhalf2(2:end)];                 %q=1.0 included
%--------------------------------------------------------------------------

[N,Q]=meshgrid(n_query,q_query);                %each column in N has const speed
                                                %each row in Q has const flow
F_ETA=scatteredInterpolant(n,q,eta,'natural');  %create interpolant that fits a surface on the form eta=F_ETA(n,q)       
ETA=F_ETA(N,Q);                                 %evaluate F_ETA at query points specified by N,Q matrices

for i=3:length(n_query-2)
    ETA(:,i)=smooth(ETA(:,i));                  %comment out for raw data
end

elev=[linspace(0,0.8,17) linspace(0.825,1.00,8)];
%elev=0:de:1;

figure(3)
contour(N,Q,ETA,elev,'-','LineWidth',1)
legend('$\tilde{\eta}$')
colorbar
caxis([0 1])
xlabel('$\tilde{\omega} / \sqrt{h}$ [-]','Interpreter','latex','FontSize',16)
ylabel('$q / \sqrt{h}$ [-]','Interpreter','latex','FontSize',16)
title('Measurements: Hill diagram','Interpreter','latex','FontSize',16)
hold on

[Y,N]=meshgrid(y_query,n_query);                %each column in Y has const opening degree
                                                %each row in N has const speed
F_Q=scatteredInterpolant(y,n,q,'natural');      %create interpolant that fits a surface on the form q=F_Q(y,n)
Q=F_Q(Y,N);                                     %evaluate F_Q at query points specified by Y,N matrices

for i=1:length(y_query)                         %plot one curve per opening degree
    Q(3:end-2,i)=smooth(Q(3:end-2,i));          %comment out for raw data
    txt=['y = ',num2str(y_query(i))];
    plot(N(2:end-1,i),Q(2:end-1,i),'-','LineWidth',1,'DisplayName',txt);  %exclude lowest and highest speed and flow from plot
end

set(legend,'Interpreter','latex','FontSize',10,'Location','west')
axis([0 1.9 0 1.8])
hold off
legend show
